library json_value;

import 'dart:convert';

enum JsonType { isMap, isList, isString, isNum, isBool, isNull, unknown }

class JsonValue {
  final dynamic _value;
  JsonValue(this._value);
  JsonType get type {
    var t = JsonType.unknown;
    if (_value is Map) {
      t = JsonType.isMap;
    } else if (_value is List) {
      t = JsonType.isList;
    } else if (_value is String) {
      t = JsonType.isString;
    } else if (_value is num) {
      t = JsonType.isNum;
    } else if (_value is bool) {
      t = JsonType.isBool;
    } else if (_value == null) {
      t = JsonType.isNull;
    }
    return t;
  }

  ///key is int or String
  JsonValue operator [](dynamic key) {
    if (key is int) {
      if (type == JsonType.isList) {
        return (key < listValue.length && key >= 0)
            ? listValue[key]
            : _nullValue;
      } else {
        return _nullValue;
      }
    } else if (key is String) {
      if (type == JsonType.isMap) {
        return mapValue.keys.contains(key) ? mapValue[key] : _nullValue;
      } else {
        return _nullValue;
      }
    }
    return _nullValue;
  }

  Map<dynamic, JsonValue> get mapValue {
    if (type == JsonType.isMap) {
      Map<String, JsonValue> map = {};
      (_value as Map<dynamic, dynamic>).forEach((key, value) {
        map[key] = JsonValue(value);
      });
      return map;
    } else {
      print('JsonValue error: value is not map');
      return null;
    }
  }

  Map<String, dynamic> get mapObject {
    if (type == JsonType.isMap) {
      return _value as Map<String, dynamic>;
    } else {
      print('JsonValue error: value is not map');
      return null;
    }
  }

  List<JsonValue> get listValue {
    if (type == JsonType.isList) {
      List<JsonValue> list = [];
      (_value as List<dynamic>).forEach((element) {
        list.add(JsonValue(element));
      });
      return list;
    } else {
      print('JsonValue error: value is not list');
      return null;
    }
  }

  List<dynamic> get listObject {
    if (type == JsonType.isList) {
      return _value as List<dynamic>;
    } else {
      print('JsonValue error: value is not list');
      return null;
    }
  }

  String get stringValue {
    if (type == JsonType.isString) {
      return _value as String;
    } else {
      print('JsonValue error: value is not string');
      return null;
    }
  }

  num get numValue {
    if (type == JsonType.isNum) {
      return _value as num;
    } else {
      print('JsonValue error: value is not num');
      return null;
    }
  }

  bool get booValue {
    if (type == JsonType.isBool) {
      return _value as bool;
    } else {
      print('JsonValue error: value is not bool');
      return null;
    }
  }

  String toJsonString() {
    return json.encode(_value);
  }

  static JsonValue _nullValue = JsonValue(null);
}
